package com.cneeds.rvhelpers.helpers;

import com.cneeds.rvhelpers.entity.IHeader;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;


/**
 * 按键/旋钮事件处理
 * author: maiqh
 */
public class KeyEventHelper {

    // 存储"坐标"对应"adapter position"的二维表结构，
    private ArrayList<ArrayList<Integer>> mCoordinationList;

    public final static int ERROR_POSITION = -1;

    // 存储"adapter position"对应"坐标"的map缓存
    private HashMap<Integer, KeyEventHelper.PositionBean> mPositionMap;
    // The number of columns in the grid
    protected int mSpanCount;
    // 缓存原始数据，用于判断需不需要更新坐标二维表
    private ArrayList mTempArrayList = new ArrayList();

    public KeyEventHelper(List<?> dataList, int spanCount) {
        mCoordinationList = new ArrayList<>();
        mPositionMap = new HashMap<>();
        mSpanCount = spanCount;
        resetCoordination(dataList);
    }

    public void resetCoordination(List<?> dataList) {

        if (dataList == null || mTempArrayList.equals(dataList)) {
            return;
        }

        mCoordinationList.clear();
        mPositionMap.clear();
        mTempArrayList.clear();
        Object lastO = null;
        for (int i = 0; i < dataList.size(); i++) {
            Object currentO = dataList.get(i);
            // 把position转换为坐标二维表
            convert2Coordination(currentO, lastO, i);
            lastO = currentO;
            mTempArrayList.add(currentO);
        }
    }

    /**
     * 把GridLayout的网格布局记录为一个坐标二维表
     *
     * @param currentT 当前元素
     * @param lastT    上一个记录的元素
     * @param adapterPosition 当前元素对应的Adapter位置
     */
    private void convert2Coordination(Object currentT, Object lastT, Integer adapterPosition) {
        if (currentT instanceof IHeader) {
            // 标题不记录
            return;
        }
        int size = mCoordinationList.size();
        // 非第一行 && 当前行不满最大列数 && 上一个item非标题
        if (size > 0 && mCoordinationList.get(size - 1).size() < mSpanCount && !(lastT instanceof IHeader)) {
            // 记录位置
            mCoordinationList.get(size - 1).add(adapterPosition);
        } else {
            // (第一行 || 当前行满最大列数 || 上一个item非标题) 满足这个三个条件其中一个都新建一行
            ArrayList<Integer> newColumnList = new ArrayList<>();
            newColumnList.add(adapterPosition);
            mCoordinationList.add(newColumnList);
        }
        // 记录position为key,对应的XY为Value的map
        mPositionMap.put(adapterPosition, new KeyEventHelper.PositionBean(mCoordinationList.get(mCoordinationList.size() - 1).size() - 1, mCoordinationList.size() - 1, adapterPosition));
    }

    /**
     * 获取当前第x列，第y行的元素的adapter位置（adapter的数据包含标题）
     *
     * @param x 第x列
     * @param y 第y行
     * @return 元素在adapter中的位置， 如果不存在就返回 {@BaseGridRecyclerAdapter.NONE_SELECT}
     */
    public int getAdapterPosition(int x, int y) {
        if (mCoordinationList.size() <= y || y < 0) {
            return ERROR_POSITION;
        }
        ArrayList<Integer> rowList = mCoordinationList.get(y);
        if (rowList == null || rowList.size() <= x) {
            return ERROR_POSITION;
        }

        Integer columnCount = rowList.get(x);
        return columnCount;
    }

    /**
     * 高亮框框移动到"合法"的 第x列，第y行 时, 直接getAdapterPosition返回对应的adapter位置
     * <p>
     * 高亮框框移动到"非法"的 第x列，第y行 时，
     * 1. x方向，移动到最后一列再往右移动，就把高亮框框移动到下一行的第一列
     * 2. y方向，上下移动时，上一行或者下一行的当前位置没有item时，移动到上一行或者下一行的最后一个位置
     *
     * @param x                第x列
     * @param y                第y行
     * @param highListPosition 当前高亮位置
     * @return 第x列，第y行 对应的adapter位置， 如果不存在
     */
    public KeyEventHelper.PositionBean moveToNext(int x, int y, int highListPosition) {
        if (mCoordinationList.size() <= y) {
            return new KeyEventHelper.PositionBean(x, y, ERROR_POSITION);
        }

        int adapterPosition = getAdapterPosition(x, y);
        ArrayList<Integer> rowList = mCoordinationList.get(y);

        // 注意，这里是为了应对第一次进入列表情况，可能不能适应所有情况
        if (highListPosition == -1 && y == 0) {
            return new KeyEventHelper.PositionBean(0, 0, 1);
        }

        if (adapterPosition == ERROR_POSITION && rowList != null && highListPosition >= 0 &&  mPositionMap.containsKey(highListPosition)) {
            KeyEventHelper.PositionBean lastPosition = mPositionMap.get(highListPosition);
            // （x方向上向右移动 && 移动后的x超过的当前行的长度）
            if (lastPosition.getX() != x && rowList.size() <= x) {
                if (lastPosition.getX() < x) {
                    /** 1.1 x方向，移动到最后一列再往右移动，就把高亮框框移动到下一行的第一列 */
                    x = 0;
                    y += 1;
                } else {
                    /** 1.2 x方向，移动到第一列再往左移动，就把高亮框框移动到上一行的最后一列 */
                    // 这里目前是走不到的，notouchsupport目前x小于0会跳到导航栏
                    x = rowList.size() - 1;
                    y -= 1;
                }
                // （y方向上或向下移动 && 移动后的x超过的当前行的长度）
            } else if (lastPosition.getY() != y && rowList.size() <= x) {
                /** 2. y方向，上下移动时，上一行或者下一行的当前位置没有item时，移动到上一行或者下一行的最后一个位置 */
                x = rowList.size() - 1;
            }
            return new KeyEventHelper.PositionBean(x, y, getAdapterPosition(x, y));
        }
        return new KeyEventHelper.PositionBean(x, y, adapterPosition);
    }

    public int getTotalCount() {
        if (mPositionMap != null) {
            return mPositionMap.size();
        } else {
            return 0;
        }
    }

    public class PositionBean {
        // 将会移动到的x轴位置
        private int x;
        // 将会移动到的y轴位置
        private int y;
        // xy对应实体在Adapter的位置
        private int adapterPosition;

        public PositionBean(int x, int y, int position) {
            this.x = x;
            this.y = y;
            this.adapterPosition = position;
        }

        public int getX() {
            return x;
        }

        public void setX(int x) {
            this.x = x;
        }

        public int getY() {
            return y;
        }

        public void setY(int y) {
            this.y = y;
        }

        public int getAdapterPosition() {
            return adapterPosition;
        }

        public void setAdapterPosition(int adapterPosition) {
            this.adapterPosition = adapterPosition;
        }
    }

}
